/*
Table of Contents:
	LV - Local Variables
	GV - Global Variables
	V - Verbs
	LP - Local Procedures
	GP - Global Procedures
	T - Turfs
	EV - Effect Variables
	L - Location Variable
	L&L - Login and Logout Procedures
Examples
	Local Variables -
		LV1:
			//Define the variable to be used in theory
			mob/var/Level

			//Example of calling the variable as usr(User), we use usr because the mob who's level we are checking is using the verb.
			verb/CheckMyLevel()
				usr << "[usr.Level]"

			//Example of calling the variable as src(Source), we use src because the proc used was derived from the mob, which is the
			Source.
			proc/CheckLevel()
				src << "[src.Level]"

			//Example of calling the variable as a pre-determined variable. In this case we will be using M as the variable.
			Example 1 -
				for(var/mob/M in world) //this will run through all the mob in your world.
					M << "[M.Level]" // this will output M's level to M. M will be whatever mob M is set to

			Example 2 -
				verb/CheckMyLevel()
					var/M = usr //Set M to equal the usr.
					M << "[M.Level]" //Display M's Level to M.

		LV2:
			mob
				var //Anything past this point will be a variable.
					Level = 0 // This will set the initial level of mob to 0. If this was a game it would make more sense to set
					the starting level to 1.

		LV3:
			//Define the e-mail var
			mob
				var
					Email

			//Make a verb to change the users e-mail var if needed.
			verb/MyEmailIs(T as text)
				usr.Email = T

		LV4:
			//Lists are practically Arrays, just simplier. They are defined the same way as any var.
			mob
				var
					list
						HouseLocations = list("Bensalem","Italy","Paris")
						/*
						These can be accesed the same way Arrays are in Java -
						HouseLocation[1] = Bensalem
						HouseLocation[2] = Italy
						HouseLocation[3] = Paris
						*/


	Global Variables -
		GV1:
			global variables can be defined two different ways.

			Example 1 -
				world
					var
						ThisIsAGlobalVariable

			Example 2 -
				var
					ThisIsAGlobalVariable

	Verbs -
		V1:
			mob/verb //Everything past this point will be attached to all mobs.

		V2:
			//Define vars you would need in theory
			mob
				var
					Equipped = "Nothing" //Define the mob's basic "Equipped" var as nothing.

			obj/sword //Create a sword Object
				verb	//Define everything past this point as a verb
					Equip()	//Define the verb to equip the item
						usr.Equipped = src.name	//set the user's(The mob who is using the verb) Equipped Var to the sources(The object the verb
						//belongs to) name.
					Unequip()	//Define the verb to unequip the item
						usr.Equipped = "Nothing" //set the user's Equipped var to nothing.

		V3:
			//Define vars you would need in theory
			turf	//define turfs
				water	//define a water turf
					CanDelete = 1	//define the water turf's CanDelete variable to 1
				grass	//define a grass turf
					CanDelete = 0	//define the grass turf's CanDelete variable to 0

				verb //Define turf verbs
					Delete()	//define the delete verb
						if(src.CanDelete == 1)	//check if the turf's(the source) CanDelete var is 1.
							del(src)	//if it's CanDelete var is 1, delete the source.
						else	//if the turf's CanDelete var is not 1
							return	//return and do nothing.

		V4:
			//Ill show you how to define an area verb even though we wont be using areas in this demo, if not in any of them.
			area	//define areas
				Fort_One	//define a Fort_One area
					Weather = "Rain"	//define it's Weather variable as Rain
				Fort_Two	//define a Fort_Two area
					Weather = "Snow"	//define it's Weather variable as snow

				verb	//define area verbs
					ChangeWeather()	//define the ChangeWeather Verb.
						if(src.Weather == "Rain")	//if the source's(The area is the source) Weather variable is rain
							src.Weather = "Snow"	//change it's Weather variable to snow
						else	//if its not equal to rain
							src.Weather = "Rain"	//change it to rain

		V5:
			mob/verb //Everything past this point will be attached to all mobs.

			mob/Owner/verb // Everything past this point will be attached to all mob's defined as Owners.

			//In this case when you log in, your defined as /mob, you will recieve all the commands under mob/verb, but not those under
			// mob/Owner/verb.


	Local Procedures -
		LP1:
			//define what you would need in theory.
			mob
				Name
				Last_Name

			mob/verb
				Who_Display()	//define the Who_Display verb
					usr.DisplayWho()	//call the user's(The mob using the verb) DisplayWho proc.

			mob/proc	//define proc past this
				DisplayWho()	//define the DisplayWho proc
					var/PeopleOnline = 0	//create a variable for this proc that will contain the amount of people online.
					src << "Who List:"	//output the title of the who list to the src(The person who activated the proc).
					for(var/mob/M in world)	//Loop through the amount of mobs in the world.
						PeopleOnline += 1	//For every mob found add one to PeopleOnline src.
						src << "[PeopleOnline] - [M.Name] [M.Last_Name]"	//output the found mob's first and last name to the src.
					src << "[PeopleOnline] People Online."	//after the for loop output how many people are onlien to the src.

		LP2:
			//define what you would need in theory
			mob
				HP = 100	//define the mob's HP var as 100
				dead = 0	//define the mob's dead value to 0

			mob/verb	//define verbs
				Hurt_Self()	//define the Hurt_Self verb
					if(usr.dead == 1)	//check if the user is dead.
						return	//if hes dead dont let him hurt himself, just return and do nothing.
					else	//if the user is not dead
						usr.HP -= 10	//subtract 10 from the user's(The one using the verb) HP var.
						usr.DeathCheck()	//Call the DeathCheck() proc for the user

			mob/proc	//define procs
				DeathCheck()	//define the DeathCheck proc
					if(src.HP <= 0)	//check if the source's(The one who the proc was called on) HP var is less than or equal to 0)
						src.dead = 1	//set his dead value to 1
						spawn(100)	//wait 10 seconds, with spawn(Value) the Value is in ticks(One tenth of a second).
						dead = 0	//set his dead var value to 0
						HP = 100	//set his HP var value to 100
					else	//if his HP value is not bellow or equal to 0
						return	//return and do nothing

		LP3:
			//this is a bit more advanced -
			//define everything you would need

			var/list/CachedIcons=list()	//we'll use this to help reduce cpu usage from the rgb()ing of icons

			obj/Supplemental/DamageNum
				icon='DamageNums.dmi'
				density=0
				layer=5	//keeps it above the players and enemies
				New(var/r,var/g,var/b)
					spawn(10) del src	//auto delete just in case!
					//Cache the icons to reduce cpu usage
					for(var/v in CachedIcons)	if(v=="Damage[r][g][b]")
						src.icon=CachedIcons[v];return
					src.icon+=rgb(r,g,b)
					CachedIcons+="Damage[r][g][b]"
					CachedIcons["Damage[r][g][b]"]=src.icon

			mob/proc/DamageShow(var/damage,var/VarR=200,VarG=0,VarB=0)	//input the amount, then rgb values to change color
				if(istext(damage))	//if your send the proc something like Block it will display that instead of #s
					var/obj/O=new/obj/Supplemental/DamageNum	//creates the new object
					O.loc=locate(src:x,src:y,src:z)	//locates it at the damaged person
					flick("[damage]",O)	//this flashes the Block icon state, or whichever u sent
					return	//then return because we dont want to show numbers
				damage=min(999999,damage)	//999,999 is the highest number BYOND can accurately track
				damage=num2text(damage)	//this converts the number to text so we can chop it up
				var/pxplus=-7	//offsets the damage by -7 pixels
				var/spot=0	//used for cutting out the text
				while(pxplus<(length(damage)*7)-7)	//this makes it show itll only display a max of 7 numbers
					spot+=1	//determines where we're cutting
					pxplus+=7	//bump the pixel display by 7 each # so they dont overlap
					var/obj/Supplemental/DamageNum/O=new()	//create the new number
					O.pixel_x+=pxplus	//offset the numbers pixel display
					O.loc=locate(src:x,src:y,src:z)	//locate it at the person being hurt
					O.icon+=rgb(VarR,VarG,VarB)	//changes the color of the damage so we can use the same icon for healing, criticals, etc
					flick("[copytext(damage,spot,spot+1)]",O)	//this flashes the cut number of damage for this obj


	Global Procedures -
		GP1:
			mob/verb	//define mob verbs
				Say_Number_Under_1000(T as text)	//define Say_Number_Under_1000, this will make you enter a message that will be set as the
					variable T, variable T is defined as text.
					var/IsFormat = CheckNumber(T)	//Call the proc CheckNumber with the arguement T, to check if T is properly formated. Save
					what is returned as IsFormat.
					if(IsFormat == 1)	//If IsFormat equals 1
						world << "[T]	//Output the text value of T to the world
					else	//if its not equal to 1
						return	//return and do nothing

			proc //define global procs
				CheckNumber(var/ToCheck)	//Define the CheckNumber proc can have it be abel to be passed an arguement which is saved as "ToCheck"
					ToCheckNumerical = text2num(ToCheck) //Create a second value to contain a numerical value of ToCheck, instead of text based
					if(!isnum(ToCheckNumerical))	//Check	if the new numerical var doesnt only contains number
						return 0	//if it doesnt, return a value of 0
					else	if(ToCheckNumerical > 1000)	//Check if the new numerical var is over 1000
						return 0	//if it is, return a value of 0
					else	//if neither of the previous conditions are met
						return 1	//return a value of 1
		GP2:
			proc	//define global procs
				RespawnWorld()	//Define the RespawnWorld proc
					world.Repop()	//this built in BYOND proc will respawn anything from runtime that was deleted
					//runtime is when the game is started (when the files are first run)
					//basicaly anything thats in the game when it first starts, excludes anything added during gameplay)
					spawn(600)	RespawnWorld()	//this will call the proc again after 60 seconds
					//because this proc calls itself it will loop for eternity! or at least until the game dies

		GP3:
			proc	//define global procs
				ListCheck(var/Object,var/List)	//look for Object in List
					if(Object in List)	return 1	//if found return 1
					else return 0	//otherwise return a 0

	Turfs -
		T1:
			turf //Define turf, everything past this point will be derived from /turf
				Nature_Turfs	//Define the nature turf directory, this is just to keep things more organized.
					icon = 'Nature_Icons.dmi'	//Set all nature turf's default icon to be 'Nature_Icons.dmi'
					Grass	//define a Grass turf
						icon_state = "Grass"	//set the Grass turf's icon_state to "Grass"
					Tree	//define a Tree turf
						icon_state = "Tree"	//set the Tree turf's icon_state to "Tree"
						density = 1	//set the Tree turf's density to 1. This will make it so mobs with a density of 1(the mob default), cannot
									//pass through it.
				Structure_Turfs	//Define the structure turf directory.
					icon = 'Structure_Icons.dmi'	//Set all structure turf's default icon to be 'Structure_Icons.dmi'
					Building	//define a building turf
						icon_state = "Building"	//set the building's icon_state to "Building". The normal size of a building would be larger
												//than that of a normal grass tile, however this is just an example.
						density = 1	//set the Building turf's density to 1. This will make it so mobs with a density of 1(the mob default), cannot
									//pass through it.

	Effect Variables -
		EV1 - EV2 - EV3:
			//Effect variables on mobs
			mob	//anything past this point is derived from /mob
				density = 0	//this overwirtes the default density of a mob(which is 1), so the mob can now through any object/turf/mob.

			//Effect variables on objects
			obj	//define everything past this point to be derived from /obj
				density = 1	//this item will not pass through a dense object.
				layer = 5	//anything bellow layer 5, on a map's Z level, will appear under this object.

			//Effect variables on turfs
			turf	//define everything past thei point to be derived from /turf
				density = 1	//things that are dense cannot move through this turf

	Loc(Location) Variable -
		L1:
			mob/verb	//Define everything past this point as a verb derived from /mob
				Move_Loc()	//Define a Move_Loc() verb
					var/XToMoveTo = rand(1,10)	//Once used make the verb generate a number of 1 to 10 using the built in rand(MinNumber, MaxNumber)
												//proc and save it as a variable called XToMoveTo
					var/YToMoveTo = rand(1,10)	//then make the verb generate a number of 1 to 10 using the rand proc and save
												//it as a variable called YToMoveTo
					usr.loc = locate(XToMoveTo, YToMoveTo, usr.z)	//use the built in locate(XToFind, YToFind, ZToFind) proc to locate the tile
																	//that you will be moving to, then	set the usr's(The user of the verb)
																	//loc(Location) var to it.

	Login and Logout procedures -
		L&L1:
			//Define everything youll need in theory
			var/list/CantRelogYet = list()	//define a global list containing keys that cant relog yet.

			proc/AddToRelog(var/key2add)	//define a global proc called "AddToRelog" with key2add as the arguement
				CanRelogYet += key2add	//add key2add to the cant relog yet list
				spawn(150)	//wait 15 seconds(spawn is called in ticks)
					CanRelogYet -= key2add	//Delete key2add from the cant relog list yet

			mob	//Derived everything past this from /mob
				Login()	//define the Login() proc
					if(src.key in CantRelogYet)	//check if the src's(The mob logging in) key is in the CantRelogYet list.
						src << "You cannot relog yet"	//if it is display a message telling them they cant relog yet.
						del src	//disconnect the source.
					..()	//if they didnt disconnect, they arent in the list, then continue with the default Login() procedure
				Logout()	//define the Logout() proc
					if(!src.key in CantRelogYet)	//check if src.key is already in the CantRelogYet list
						CantRelotYet += src.key	//if it is not, add the src's(The mob logging out) key to the CantRelogYet list.
					src del	//delete the source
					..()	//continue with the default Logout() procedure

		L&L2:
			mob/Login()//Define a mob's Login() proc
				world << "Info Bot: [src.key] has logged in!"	//When a mob logs in have it display that they have entered the
																//game
				..()	//This makes the rest of the mob's "Login" proc run, seeing as it has built in procedures.

			mob/Logout()
				world << "Info Bot: [src.key] has logged in!"	//When a mob logs out have it display that they have left the
																//game
				..()	//This makes the rest of the mob's "Logout" proc run, seeing as it have built in procedures.

		L&L3:
			//define everything needed in theory
			var/list/BanList = list("ThisKeyIsBanned")	//define a global list called "BanList" and put the key "ThisKeyIsBanned" inside it.

			mob/Login()	//define the Login() proc
				if(src.key in BanList)	//check if the src's(The mob logging in) key is in the BanList
					src << "You are banned."	//if it is display the message that they are banned.
					del src	//then delete the source
				..()	//is they didnt disconnect they are not in the BanList, and are not banned, continue with the default Login() procedure.




*/